<?php

class WpakBuild {

	const export_file_memory = 10;

	public static function hooks() {
		if ( is_admin() ) {
			add_action( 'wp_ajax_wptm_build_app_sources', array( __CLASS__, 'ajax_build_app_sources' ) );
			add_action( 'wp_ajax_wptm_create_phonegap_app', array( __CLASS__, 'ajax_create_phonegap_app' ) );
			add_action( 'wp_ajax_wptm_create_keystore_android_app', array( __CLASS__, 'ajax_create_keystore_android_app' ) );
			add_action( 'wp_ajax_wptm_create_keystore_ios_app', array( __CLASS__, 'ajax_create_keystore_ios_app' ) );
			add_action( 'admin_action_wptm_download_app_sources', array( __CLASS__, 'download_app_sources' ) );
			add_action( 'add_meta_boxes', array( __CLASS__, 'add_meta_boxes' ), 20 );
			add_action( 'save_post', array( __CLASS__, 'save_post' ) );
		}
	}

	public static function add_meta_boxes() {
		add_meta_box(
			'wptm_simulation_box',
			__( 'Dev Tools', WpTomob::i18n_domain ),
			array( __CLASS__, 'inner_simulation_box' ),
			'wptm_apps',
			'side',
			'default'
		);
		
	}

	public static function inner_simulation_box( $post, $current_box ) {
		$debug_mode = self::get_app_debug_mode_raw( $post->ID );
		?>
		<a href="#" class="hide-if-no-js wptm_help"><?php _e( 'Help me', WpTomob::i18n_domain ); ?></a>
		<div class="field-group">
			<label><?php _e( 'Debug Mode', WpTomob::i18n_domain ) ?></label>
			<select name="wptm_app_debug_mode">
				<option value="on" <?php echo $debug_mode == 'on' ? 'selected="selected"' : '' ?>><?php _e( 'On', WpTomob::i18n_domain ) ?></option>
				<option value="off" <?php echo $debug_mode == 'off' ? 'selected="selected"' : '' ?>><?php _e( 'Off', WpTomob::i18n_domain ) ?></option>
				<option value="wp" <?php echo $debug_mode == 'wp' ? 'selected="selected"' : '' ?>><?php _e( 'Same as WordPress WP_DEBUG', WpTomob::i18n_domain ) ?></option>
			</select>
			<span class="description"><?php _e( 'If activated, echoes debug information in the browser JavasSript console while simulating the app.', WpTomob::i18n_domain ) ?></span>
		</div>
		<div class="field-group">
			<a href="<?php echo esc_url( self::get_appli_dir_url() . '/config.js?wptm_app_id=' . WpakApps::get_app_slug( $post->ID ) ) ?>" target="_blank"><?php _e( 'View config.js', WpTomob::i18n_domain ) ?></a>
		</div>
		<?php wp_nonce_field( 'wpak-simulation-data-' . $post->ID, 'wpak-nonce-simulation-data' ) ?>
		<?php
		do_action( 'wptm_inner_simulation_box', $post, $current_box );
	}

	public static function save_post( $post_id ) {

		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}

		if ( empty( $_POST['post_type'] ) || $_POST['post_type'] != 'wptm_apps' ) {
			return;
		}

		if ( !current_user_can( 'edit_post', $post_id ) && !current_user_can( 'wptm_edit_apps', $post_id ) ) {
			return;
		}

		if ( !check_admin_referer( 'wpak-simulation-data-' . $post_id, 'wpak-nonce-simulation-data' ) ) {
			return;
		}

		if ( isset( $_POST['wptm_app_debug_mode'] ) ) {
			update_post_meta( $post_id, '_wptm_app_debug_mode', $_POST['wptm_app_debug_mode'] );
		}
	}

	private static function get_app_debug_mode_raw( $app_id ) {
		$debug_mode = get_post_meta( $app_id, '_wptm_app_debug_mode', true );
		return empty( $debug_mode ) ? 'off' : $debug_mode;
	}

	public static function get_app_debug_mode( $app_id ) {
		$debug_mode = self::get_app_debug_mode_raw( $app_id );
		return $debug_mode == 'wp' ? (WP_DEBUG ? 'on' : 'off') : $debug_mode;
	}

	public static function get_appli_dir_url() {
		return plugins_url( 'app', dirname( dirname( __FILE__ ) ) );
	}

	public static function get_appli_index_url( $app_id ) {
		return self::get_appli_dir_url() . '/index.html?wptm_app_id=' . WpakApps::get_app_slug( $app_id );
	}

	public static function get_allowed_export_types() {
		$allowed_export_types = array(
			'phonegap-build' => __( 'PhoneGap Build', WpTomob::i18n_domain ),
			'phonegap-cli' => __( 'PhoneGap CLI', WpTomob::i18n_domain ),
			'webapp' => __( 'WebApp', WpTomob::i18n_domain )
		);
		return $allowed_export_types;
	}

	public static function is_allowed_export_type( $export_type ) {
		return array_key_exists( $export_type, self::get_allowed_export_types() );
	}

	public static function download_app_sources() {

		if ( !check_admin_referer( 'wptm_download_app_sources' ) || !isset( $_GET['post'] ) ) {
			return;
		}

		$app_id = intval( $_GET['post'] );

		if( $app_id <= 0 ) {
			return;
		}

		$export_type = isset( $_GET['export_type'] ) && self::is_allowed_export_type( $_GET['export_type'] ) ? $_GET['export_type'] : 'phonegap-build';

		// Re-build sources
		$answer = self::build_app_sources( $app_id, $export_type );

		if( 1 === $answer['ok'] && !empty( $answer['export'] ) ) {
			$filename = $answer['export'] . '.zip';
			$filename_full = self::get_export_files_path() . "/" . $filename;
		}
		else {
			echo esc_html( $answer['msg'] );
			exit;
		}

		if ( file_exists( $filename_full ) ) {

			header( "Pragma: public" );
			header( "Expires: 0" );
			header( "Cache-Control: must-revalidate, post-check=0, pre-check=0" );
			header( "Cache-Control: public" );
			header( "Content-Description: File Transfer" );
			header( "Content-type: application/octet-stream" );
			header( "Content-Disposition: attachment; filename=\"" . $filename . "\"" );
			header( "Content-Transfer-Encoding: binary" );
			header( "Content-Length: " . filesize( $filename_full ) );
			ob_end_clean();
			@readfile( $filename_full );
			exit;
		} else {
			echo sprintf( __( 'Error: Could not find zip export file [%s]', WpTomob::i18n_domain ), $filename_full );
			exit;
		}
	}

	/**
	* Retrieves export data and launch zip built.
	*
	* @param int		$app_id				Application id
	* @param string     $export_type        Export type : 'phonegap-build' (default), 'phonegap-cli' or 'webapp'
	*/
	public static function build_app_sources( $app_id, $export_type = 'phonegap-build' ) {
		$answer = array();

		if ( !self::is_allowed_export_type( $export_type ) ) {
			$answer['ok'] = 0;
			$answer['msg'] = __( 'Unknown export type', WpTomob::i18n_domain ) . ": '$export_type'";
			return $answer;
		}

		if ( !extension_loaded( 'zip' ) ) {
			$answer['ok'] = 0;
			$answer['msg'] = __( 'Zip PHP extension is required to run file export. See http://www.php.net/manual/fr/book.zip.php.', WpTomob::i18n_domain );
			return $answer;
		}

		if ( !self::create_export_directory_if_doesnt_exist() ) {
			$export_directory = self::get_export_files_path();
			$answer['ok'] = 0;
			$answer['msg'] = sprintf( __( 'The export directory [%s] could not be created. Please check that you have the right permissions to create this directory.', WpTomob::i18n_domain ), $export_directory );
			return $answer;
		}

		//If the app current theme has some PHP (hooks!) to be executed before
		//we build the export, include it here :
		WpakThemes::include_app_theme_php( $app_id );
		
		//Include PHP files required by addons activated for this app :
		WpakAddons::require_app_addons_php_files( $app_id );
		
		$current_theme = WpakThemesStorage::get_current_theme( $app_id );

		$plugin_dir = plugin_dir_path( dirname( dirname( __FILE__ ) ) );
		$appli_dir = $plugin_dir . 'app';
		WpakApps::get_app_color_setup( $app_id,$appli_dir);
		$export_filename = self::get_export_file_base_name( $app_id );
		$export_filename_full = self::get_export_files_path() . "/" . $export_filename . '.zip';

		$app_main_infos = WpakApps::get_app_main_infos( $app_id );
		$app_platform = $app_main_infos['platform'];

		$answer = self::build_zip(
				$app_id,
				$appli_dir,
				$export_filename_full,
				array( $current_theme ),
				WpakAddons::get_app_addons( $app_id ),
				WpakConfigFile::get_platform_icons_and_splashscreens_files( $app_id, $app_platform, $export_type ),
				$export_type
		);

		$answer['export'] = $export_filename;
		$answer['export_full_name'] = $export_filename_full;

		return $answer;
	}
public static function ajax_create_phonegap_app(){
	global $wpdb;
	$main_infos = WpakApps::get_app_main_infos( $_POST['postid'] );
	if (!empty($_POST['file'])) {
	# code...
		ob_start();
		$file_name_with_full_path=$_POST['file'];
if (function_exists('curl_file_create')) { // php 5.5+
  $cFile = curl_file_create($file_name_with_full_path);
} else { // 
  $cFile = '@' . realpath($file_name_with_full_path);
}
$platform=$main_infos['platform'];
$application =  $wpdb->get_results("
													      SELECT
													        *
													      FROM
													        {$wpdb->prefix}wptm_keystore_table
													      WHERE platform='$platform'
													      ORDER BY id DESC
    												");
if (count($application)!=0){
	$androidkey_id=0;
	$ioskey_id=0;
	foreach ($application as $key => $value) {
		# code...
	if ($value->platform=='android') {
		# code...
	
	
								$androidkey_id=$value->key_id;
								$key_password=$value->key_password;
								$keystore_pw=$value->alais_password;
        $options = array(
            'key_pw' => $key_password,
            'keystore_pw' => $keystore_pw,

        );

            // extract files, pass them separately
            $files = array();
           
            $data = $files;
            if (! empty($options)) {
                // original API requires data to be in json format
                $data = array_merge($data, array(
                    'data' => json_encode($options),
                ));
            }


								$ch = curl_init();
curl_setopt($ch, CURLOPT_URL,'https://build.phonegap.com/api/v1/keys/android/'.$androidkey_id.'/?access_token='.get_option('wptm_phonegap_access_token'));
curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
curl_setopt($handle, CURLOPT_HTTPHEADER, array(
            'Content-type: multipart/form-data;',
        ));
curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
curl_exec ($ch);
curl_close ($ch);
		} else {
		# code...

	
								$ioskey_id=$value->key_id;
								$key_password=$value->key_password;
        $options = array(
            'password' => $key_password,

        );

            // extract files, pass them separately
            $files = array();
           
            $data = $files;
            if (! empty($options)) {
                // original API requires data to be in json format
                $data = array_merge($data, array(
                    'data' => json_encode($options),
                ));
            }


								$ch = curl_init();
curl_setopt($ch, CURLOPT_URL,'https://build.phonegap.com/api/v1/keys/ios/'.$ioskey_id.'/?access_token='.get_option('wptm_phonegap_access_token'));
curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
curl_setopt($handle, CURLOPT_HTTPHEADER, array(
            'Content-type: multipart/form-data;',
        ));
curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
curl_exec ($ch);
curl_close ($ch);
	}		

}
ob_end_clean();  
		ob_start();		

$postid=$_POST['postid'];
		$main_infos = WpakApps::get_app_main_infos( $postid );
$platform=$main_infos['platform'];
$keys = ($platform=='android') ? "\"android\":{\"id\":$androidkey_id}" : "\"ios\":{\"id\":$ioskey_id}" ;
$post = array('file'=> $cFile,"data"=>"{\"create_method\" : \"file\",\"keys\":{".$keys."}}");
}else{

$post = array('file'=> $cFile,"create_method"=>"file");
}
$ch = curl_init();
curl_setopt($ch, CURLOPT_URL,'https://build.phonegap.com/api/v1/apps?access_token='.get_option('wptm_phonegap_access_token'));
curl_setopt($ch, CURLOPT_POST,1);
curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, FALSE );
curl_exec ($ch);
curl_close ($ch);
$result= ob_get_contents();

ob_end_clean();
// echo $result;
$main_infos = WpakApps::get_app_main_infos( $_POST['postid'] );
$obj=json_decode( $result);
$postid=$_POST['postid'];
		$main_infos = WpakApps::get_app_main_infos( $postid );
$icon=$main_infos['wptm_icon'];
$appids=$obj->id;
$title=$obj->title;
$description=$obj->description;
$version=$obj->version;
$platform=$main_infos['platform'];
//echo "INSERT INTO {$wpdb->prefix}wptm_phonegap_table( 'app_id', 'post_id', 'title', 'description',  'icon', 'version') VALUES ($appids,$postid,'$title','$description','$icon','$version')";
 $application =  $wpdb->get_results("
      SELECT
        *
      FROM
        {$wpdb->prefix}wptm_phonegap_table
      WHERE post_id = '$postid' AND version='$version' AND platform=$platform
      ORDER BY id DESC
    ");
 if (count($application)==0) {
 	# code...

$wpdb->query("INSERT INTO {$wpdb->prefix}wptm_phonegap_table( app_id, post_id, title, description,  icon, version,platform) VALUES ($appids,$postid,'$title','$description','$icon','$version','$platform')");
echo mysql_error();
if($wpdb->insert_id!=0)echo 'done';
 }else{
 	echo "done";
 }
//echo $result;
exit();
}
}
public static function ajax_create_keystore_android_app(){
	global $wpdb,$wp_upload_dir;

	if (!empty($_FILES["file"])) {
	# code...
$target_dir = wp_upload_dir();
$target_file = $target_dir['basedir'] .'/'. basename($_FILES["file"]["name"]);
$uploadOk = 1;
$imageFileType = pathinfo($target_file,PATHINFO_EXTENSION);
if($imageFileType != "keystore"  ) {
    echo "Sorry, only keystore files are allowed.";
    $uploadOk = 0;
} 
if ($uploadOk == 0) {
    echo "Sorry, your file was not uploaded.";
// if everything is ok, try to upload file
} else {
    if (move_uploaded_file($_FILES["file"]["tmp_name"], $target_file)) {
        //echo "The file ". basename( $_FILES["file"]["name"]). " has been uploaded.";
    } else {
        echo "Sorry, there was an error uploading your file.";
    }
}
		$file_name_with_full_path=$target_file;
if (function_exists('curl_file_create')) { // php 5.5+
  $cFile = curl_file_create($file_name_with_full_path);
} else { // 
  $cFile = '@' . realpath($file_name_with_full_path);
}
$title=$_POST['title'];
$alias=$_POST['alias'];
 $password=$_POST['password'];
 $alias_password=$_POST['alias_password'];
$post = array('keystore'=> $cFile,"data"=>"{\"title\":\"$title\",\"alias\":\"$alias\", \"key_pw\":\"$password\",\"keystore_pw\":\"$alias_password\"}");

		ob_start();
$ch = curl_init();
curl_setopt($ch, CURLOPT_URL,'https://build.phonegap.com/api/v1/keys/android/?access_token='.get_option('wptm_phonegap_access_token'));
curl_setopt($ch, CURLOPT_POST,1);

curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, FALSE );
curl_setopt($ch,  CURLOPT_USERAGENT ,"Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1a2pre) Gecko/2008073000 Shredder/3.0a2pre ThunderBrowse/3.2.1.8");
curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
curl_exec ($ch);
echo curl_error($ch);

curl_close ($ch);
$result= ob_get_contents();

ob_end_clean();
$obj=json_decode( $result);
$keyids=$obj->id;

//echo "INSERT INTO {$wpdb->prefix}wptm_phonegap_table( 'app_id', 'post_id', 'title', 'description',  'icon', 'version') VALUES ($appids,$postid,'$title','$description','$icon','$version')";
 /*$application =  $wpdb->get_results("
      SELECT
        *
      FROM
        {$wpdb->prefix}wptm_keystore_table
      WHERE post_id = '$postid' AND version='$version' AND platform=$platform
      ORDER BY id DESC
    ");
 if (count($application)==0) {
 	# code...*/

$wpdb->query("INSERT INTO {$wpdb->prefix}wptm_keystore_table( key_id, key_password, alais, alais_password,platform) VALUES ($keyids,'$password','$alias','$alias_password','android')");
echo mysql_error();
if($wpdb->insert_id!=0)echo 'done';
 /*}else{
 	echo "done";
 }*/
//echo $result;
exit();
}
}
public static function ajax_create_keystore_ios_app(){
	global $wpdb,$wp_upload_dir;

	if (!empty($_FILES["file"])) {
	# code...
$target_dir = wp_upload_dir();
if (!file_exists($target_dir['basedir'] .'/keystores')) {
$oldmask = umask(0);
mkdir($target_dir['basedir'] .'/keystores', 0777);
umask($oldmask);
}
$target_dir['basedir'] = $target_dir['basedir'] .'/keystores';
$target_file = $target_dir['basedir'] .'/'. basename($_FILES["file"]["name"]);
$uploadOk = 1;
$imageFileType = pathinfo($target_file,PATHINFO_EXTENSION);
if($imageFileType != "p12"  ) {
    echo "Sorry, only keystore files are allowed.";
    $uploadOk = 0;
} 
if ($uploadOk == 0) {
    echo "Sorry, your file was not uploaded.";
// if everything is ok, try to upload file
} else {
    if (move_uploaded_file($_FILES["file"]["tmp_name"], $target_file)) {
        //echo "The file ". basename( $_FILES["file"]["name"]). " has been uploaded.";
    } else {
        echo "Sorry, there was an error uploading your file.";
    }
}
		$file_name_with_full_path=$target_file;
if (function_exists('curl_file_create')) { // php 5.5+
  $cFile = curl_file_create($file_name_with_full_path);
} else { // 
  $cFile = '@' . realpath($file_name_with_full_path);
}
$target_file1 = $target_dir['basedir'] .'/'. basename($_FILES["file1"]["name"]);

$imageFileType1 = pathinfo($target_file1,PATHINFO_EXTENSION);
if($imageFileType1 != "mobileprovision"  ) {
    echo "Sorry, only keystore files are allowed.";
    $uploadOk = 0;
} 
if ($uploadOk == 0) {
    echo "Sorry, your file was not uploaded.";
// if everything is ok, try to upload file
} else {
    if (move_uploaded_file($_FILES["file1"]["tmp_name"], $target_file1)) {
        //echo "The file ". basename( $_FILES["file"]["name"]). " has been uploaded.";
    } else {
        echo "Sorry, there was an error uploading your file.";
    }
}
		$file_name_with_full_path1=$target_file1;
if (function_exists('curl_file_create')) { // php 5.5+
  $cFile1 = curl_file_create($file_name_with_full_path1);
} else { // 
  $cFile1 = '@' . realpath($file_name_with_full_path1);
}
$title=$_POST['title'];
 $password=$_POST['password'];
$post = array('cert'=> $cFile,'profile'=> $cFile1,"data"=>"{\"title\":\"$title\", \"password\":\"$password\"}");

		// ob_start();
$ch = curl_init();
curl_setopt($ch, CURLOPT_URL,'https://build.phonegap.com/api/v1/keys/ios/?access_token='.get_option('wptm_phonegap_access_token'));
curl_setopt($ch, CURLOPT_POST,1);

curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, FALSE );
curl_setopt($ch,  CURLOPT_USERAGENT ,"Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.1a2pre) Gecko/2008073000 Shredder/3.0a2pre ThunderBrowse/3.2.1.8");
curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
curl_exec ($ch);
echo curl_error($ch);

curl_close ($ch);
// $result= ob_get_contents();

// ob_end_clean();
$obj=json_decode( $result);
$keyids=$obj->id;

//echo "INSERT INTO {$wpdb->prefix}wptm_phonegap_table( 'app_id', 'post_id', 'title', 'description',  'icon', 'version') VALUES ($appids,$postid,'$title','$description','$icon','$version')";
 /*$application =  $wpdb->get_results("
      SELECT
        *
      FROM
        {$wpdb->prefix}wptm_keystore_table
      WHERE post_id = '$postid' AND version='$version' AND platform=$platform
      ORDER BY id DESC
    ");
 if (count($application)==0) {
 	# code...*/

$wpdb->query("INSERT INTO {$wpdb->prefix}wptm_keystore_table( key_id, key_password, alais, alais_password,platform) VALUES ($keyids,'$password','','','ios')");
echo mysql_error();
if($wpdb->insert_id!=0)echo 'done';
 /*}else{
 	echo "done";
 }*/
//echo $result;
exit();
}
}
	public static function ajax_build_app_sources() {
		$answer = array( 'ok' => 1, 'msg' => '' );

		if ( empty( $_POST ) || empty( $_POST['app_id'] ) || !is_numeric( $_POST['app_id'] ) ) {
			$answer['ok'] = 0;
			$answer['msg'] = __( 'Wrong application ID', WpTomob::i18n_domain );
			self::exit_sending_json( $answer );
		}

		$app_id = addslashes( $_POST['app_id'] );

		//if ( !check_admin_referer( 'wptm_build_app_sources_' . $app_id, 'nonce' ) ) {
			//return;
		//}

		$answer = self::build_app_sources( $app_id );

		self::exit_sending_json( $answer );
	}

	private static function exit_sending_json( $answer ) {
		//If something was displayed before, clean it so that our answer can
		//be valid json (and store it in an "echoed_before_json" answer key
		//so that we can warn the user about it) :
		$content_already_echoed = ob_get_contents();
		if ( !empty( $content_already_echoed ) ) {
			$answer['echoed_before_json'] = $content_already_echoed;
			ob_end_clean();
		}

		header( 'Content-type: application/json' );
		echo json_encode( $answer );
		exit();
	}

	private static function get_export_files_path() {
		$upload_dir = wp_upload_dir();
		return empty( $upload_dir['error'] ) ? $upload_dir['basedir'] . '/wptomob-export' : '';
	}

	private static function get_export_file_base_name( $app_id ) {
		return 'phonegap-export-' . WpakApps::get_app_slug( $app_id );
	}

	private static function create_export_directory_if_doesnt_exist() {
		$export_directory = self::get_export_files_path();
		$ok = true;
		if ( !file_exists( $export_directory ) ) {
			$ok = mkdir( $export_directory, 0777, true );
		}
		return $ok;
	}

	private static function build_zip( $app_id, $source, $destination, $themes, $addons, $icons_and_splashscreens, $export_type ) {

		$answer = array( 'ok' => 1, 'msg' => '' );

		if ( !extension_loaded( 'zip' ) || !file_exists( $source ) ) {
			$answer['msg'] = sprintf( __( 'The Zip archive file [%s] could not be created. Please check that you have the permissions to write to this directory.', WpTomob::i18n_domain ), $destination );
			$answer['ok'] = 0;
			return $answer;
		}

		$zip = new ZipArchive();

		//
		// ZipArchive::open() returns TRUE on success and an error code on failure, not FALSE
		// All other used ZipArchive methods return FALSE on failure
		//
		// Apparently ZipArchive::OVERWRITE is not sufficient for recent PHP versions (>= 5.2.8, cf. comments here: http://fr.php.net/manual/en/ziparchive.open.php)
		//

		if ( true !== ( $error_code = $zip->open( $destination, ZipArchive::CREATE | ZipArchive::OVERWRITE ) ) ) {
			switch( $error_code ) {
				case ZipArchive::ER_EXISTS:
					$error = _x( 'File already exists', 'ZipArchive::ER_EXISTS error', WpTomob::i18n_domain );
					break;
				case ZipArchive::ER_INCONS:
					$error = _x( 'Zip archive inconsistent', 'ZipArchive::ER_INCONS error', WpTomob::i18n_domain );
					break;
				case ZipArchive::ER_INVAL:
					$error = _x( 'Invalid argument', 'ZipArchive::ER_INVAL error', WpTomob::i18n_domain );
					break;
				case ZipArchive::ER_MEMORY:
					$error = _x( 'Malloc failure', 'ZipArchive::ER_MEMORY error', WpTomob::i18n_domain );
					break;
				case ZipArchive::ER_NOENT:
					$error = _x( 'No such file', 'ZipArchive::ER_NOENT error', WpTomob::i18n_domain );
					break;
				case ZipArchive::ER_NOZIP:
					$error = _x( 'Not a zip archive', 'ZipArchive::ER_NOZIP error', WpTomob::i18n_domain );
					break;
				case ZipArchive::ER_OPEN:
					$error = _x( 'Can\'t open file', 'ZipArchive::ER_OPEN error', WpTomob::i18n_domain );
					break;
				case ZipArchive::ER_READ:
					$error = _x( 'Read error', 'ZipArchive::ER_READ error', WpTomob::i18n_domain );
					break;
				case ZipArchive::ER_SEEK:
					$error = _x( 'Seek error', 'ZipArchive::ER_SEEK error', WpTomob::i18n_domain );
					break;
				default:
					$error = '';
			}
			$answer['msg'] = sprintf( __( 'The Zip archive file [%s] could not be opened (%s). Please check that you have the permissions to write to this directory.', WpTomob::i18n_domain ), $destination, $error );
			$answer['ok'] = 0;
			return $answer;
		}

		if ( is_dir( $source ) === true ) {

			$webapp_files = array();

			$source_root = 'www';

			if ( $export_type === 'phonegap-cli' ) {
				//PhoneGap CLI export is made in www subdirectory
				//( only config.xml stays at zip root )
				$source_root = 'www';
				if ( !$zip->addEmptyDir( $source_root ) ) {
					$answer['msg'] = sprintf( __( 'Could not add directory [%s] to zip archive', WpTomob::i18n_domain ), $source_root );
					$answer['ok'] = 0;
					return $answer;
				}
			}

			if ( !empty( $source_root ) ) {
				$source_root .= '/';
			}
			
			$files = new RecursiveIteratorIterator( new RecursiveDirectoryIterator( $source ), RecursiveIteratorIterator::SELF_FIRST );

			foreach ( $files as $file ) {
				$filename = str_replace( $source, '', $file );
				$filename = wp_normalize_path( $filename );
				$filename = ltrim( $filename, '/\\' );

				//Themes are included separately from the wpak themes directory
				if ( preg_match( '|themes[/\\\].+|', $filename ) ) {
					continue;
				}

				$zip_filename = $source_root . $filename;

				if ( is_dir( $file ) === true ) {

					if ( !$zip->addEmptyDir( $zip_filename ) ) {
						$answer['msg'] = sprintf( __( 'Could not add directory [%s] to zip archive', WpTomob::i18n_domain ), $zip_filename );
						$answer['ok'] = 0;
						return $answer;
					}

				} elseif ( is_file( $file ) === true ) {

					if ( $filename == 'index.html' ) {

						$index_content = self::filter_index( file_get_contents( $file ), $export_type );

						if ( !$zip->addFromString( $zip_filename, $index_content ) ) {
							$answer['msg'] = sprintf( __( 'Could not add file [%s] to zip archive', WpTomob::i18n_domain ), $zip_filename );
							$answer['ok'] = 0;
							return $answer;
						}

					} else {

						if ( !$zip->addFile( $file, $zip_filename ) ) {
							$answer['msg'] = sprintf( __( 'Could not add file [%s] to zip archive', WpTomob::i18n_domain ), $zip_filename );
							$answer['ok'] = 0;
							return $answer;
						}

					}

					$webapp_files[] = $zip_filename;
				}
			}
			
			//Add JS Files that must be copied from WordPress core:
			$core_js_files = array(
				'vendor/jquery.js' => ABSPATH . WPINC .'/js/jquery/jquery.js',
				'vendor/underscore.js' => ABSPATH . WPINC .'/js/underscore.min.js',
				'vendor/backbone.js' => ABSPATH . WPINC .'/js/backbone.min.js',
			);
			
			foreach( $core_js_files as $app_file => $real_file ) {
				$zip_filename = $source_root . $app_file;
				if ( !$zip->addFile( $real_file, $zip_filename ) ) {
					$answer['msg'] = sprintf( __( 'Could not add file [%s] to zip archive', WpTomob::i18n_domain ), $zip_filename );
					$answer['ok'] = 0;
					return $answer;
				}
				$webapp_files[] = $zip_filename;
			}

			//Add themes files :
			if( !empty( $themes ) ) {

				$themes_directory = WpakThemes::get_themes_directory();
				if( is_dir( $themes_directory ) ) {

					$files = new RecursiveIteratorIterator( new RecursiveDirectoryIterator( $themes_directory ), RecursiveIteratorIterator::SELF_FIRST );

					foreach ( $files as $file ) {
						$filename = str_replace( $themes_directory, '', $file );
						$filename = wp_normalize_path( $filename );
						$filename = ltrim( $filename, '/\\' );

						//Filter themes :
						$theme = preg_replace( '|([^/\\\]*)[/\\\].*|', '$1', $filename );

						if ( !in_array( $theme, $themes ) ) {
							continue;
						}

						//Filter php directory
						if ( preg_match( '|'. $theme .'[/\\\]php|', $filename ) ) {
							continue;
						}

						//Filter git files
						if ( strpos( $filename, '.git' ) !== false ) {
							continue;
						}

						$filename = 'themes/'. $filename;

						$zip_filename = $source_root . $filename;

						if ( is_dir( $file ) === true ) {
							if ( !$zip->addEmptyDir( $zip_filename ) ) {
								$answer['msg'] = sprintf( __( 'Could not add directory [%s] to zip archive', WpTomob::i18n_domain ), $zip_filename );
								$answer['ok'] = 0;
								return $answer;
							}
						} elseif ( is_file( $file ) === true ) {

							if ( !$zip->addFile( $file, $zip_filename ) ) {
								$answer['msg'] = sprintf( __( 'Could not add file [%s] to zip archive', WpTomob::i18n_domain ), $zip_filename );
								$answer['ok'] = 0;
								return $answer;
							}

							$webapp_files[] = $zip_filename;

						}
					}
				}
			}

			//Add addons files :
			if ( !empty( $addons ) ) {
				foreach ( $addons as $addon ) {
					$addon_files = $addon->get_all_files();
					foreach ( $addon_files as $addon_file ) {
						$zip_filename = $source_root .'addons/'. $addon->slug .'/'. $addon_file['relative'];
						$zip->addFile( $addon_file['full'], $zip_filename );
						$webapp_files[] = $zip_filename;
					}
				}
			}

			//Add icons and splashscreens files :
			//var_dump( !empty( $icons_and_splashscreens )
				 //&& array_key_exists( 'icons', $icons_and_splashscreens ));

			if ( !empty( $icons_and_splashscreens )
				 && array_key_exists( 'icons', $icons_and_splashscreens )
				) {

				$icons = $icons_and_splashscreens['icons'];
				foreach ( $icons as $icon ) {
					$zip_filename = $source_root . $icon['src'];
					//var_dump($icon['full_path']);
					 $zip->addFile( $icon['full_path'], $zip_filename );
				}

				$splashscreens = $icons_and_splashscreens['splashscreens'];
				foreach ( $splashscreens as $splashscreen ) {
					$zip_filename =  $splashscreen['src'];
					$zip->addFile( $splashscreen['full_path'], $zip_filename );
				}

			}

			//Create config.js file :
			$zip->addFromString( $source_root .'config.js', WpakConfigFile::get_config_js( $app_id ) );
			$webapp_files[] = $source_root .'config.js';

			if ( $export_type !== 'webapp' ) {
				//Create config.xml file (stays at zip root) :
				$zip->addFromString( 'config.xml', WpakConfigFile::get_config_xml( $app_id, false, $export_type ) );
			}

		$app_main_infos = WpakApps::get_app_main_infos( $app_id );
			if( !empty( $app_main_infos['wptm_app_google_service']  ) ) {
					$cache_manifest_content = $app_main_infos['wptm_app_google_service'] ;
				if( $app_main_infos['platform'] == 'ios' ) {
				//Create html cache manifest file
					$zip->addFromString( 'GoogleService-Info.plist', $cache_manifest_content );
				}else{
					$zip->addFromString( 'google-services.json', $cache_manifest_content );
				}
			}
			if ( $export_type === 'webapp' ) {
				//Create html cache manifest file
				$cache_manifest_content = self::get_cache_manifest_content( $webapp_files );
				$zip->addFromString( 'wpak.appcache', $cache_manifest_content );
			}

		} else {
			$answer['msg'] = sprintf( __( 'Zip archive source directory [%s] could not be found.', WpTomob::i18n_domain ), $source );
			$answer['ok'] = 0;
			return $answer;
		}

		if ( !$zip->close() ) {
			$answer['msg'] = __( 'Error during archive creation', WpTomob::i18n_domain );
			$answer['ok'] = 0;
			return $answer;
		}

		return $answer;
	}

	private static function filter_index( $index_content, $export_type ) {

		if ( $export_type === 'webapp' ) {

			//Add reference to the cache manifest to the <html> tag:
			$index_content = str_replace( '<html>', '<html manifest="wpak.appcache" >', $index_content );

			//Remove script used only for app simulation in web browser :
			$index_content = preg_replace( '/<script[^>]*>[^<]*var require[^<]*?(<\/script>)/is', '', $index_content );

		} else {

			//Add cordova.js script (set cordova.js instead of phonegap.js, because PhoneGap Developer App doesn't seem
			//to support phonegap.js). PhoneGap Build can use indifferently cordova.js or phonegap.js.
			$index_content = str_replace( '<head>', "<head>\r\n\t\t<script src=\"cordova.js\"></script>\r\n\t\t", $index_content );

		}

		//Remove script used only for app simulation in web browser :
		$index_content = preg_replace( '/<script[^>]*>[^<]*var query[^<]*<\/script>\s*<script/is', '<script', $index_content );

		return $index_content;
	}

	private static function get_cache_manifest_content( $webapp_files ) {
		$cache_manifest = '';

		if ( !empty( $webapp_files ) ) {
			$cache_manifest .= "CACHE MANIFEST\n";

			$cache_manifest .= "# v". date( 'Y-m-d H:i:s' ) ."\n\n";

			foreach( $webapp_files as $file ) {
				$cache_manifest .= $file ."\n";
			}

			$cache_manifest .= "\nNETWORK:\n";
			$cache_manifest .= "/wptomob-api\n";
			$cache_manifest .= "/wp-content/uploads\n";
		}

		return $cache_manifest;
	}

}

WpakBuild::hooks();
