<?php

class WpakThemesBoSettings {
	const menu_item = 'wptm_bo_themes';

	public static function hooks() {
		add_action( 'add_meta_boxes', array( __CLASS__, 'add_meta_boxes' ), 11 );
		add_action( 'save_post', array( __CLASS__, 'save_post' ) );
		add_action( 'admin_menu', array( __CLASS__, 'admin_menu' ) );
		add_action( 'admin_enqueue_scripts', array( __CLASS__, 'admin_enqueue_scripts' ) );
		add_filter( 'admin_body_class', array( __CLASS__, 'admin_body_class' ) );
	}

	public static function add_meta_boxes() {
		// add_meta_box(
		// 	'wptm_app_theme',
		// 	__( 'Appearance', WpTomob::i18n_domain ),
		// 	array( __CLASS__, 'inner_main_infos_box' ),
		// 	'wptm_apps',
		// 	'normal',
		// 	'default'
		// );
	}

	public static function admin_menu() {
		$capability_required = current_user_can( 'wptm_edit_apps' ) ? 'wptm_edit_apps' : 'manage_options';
		add_submenu_page(
			WpakApps::menu_item,
			__( 'wptomob Themes', WpTomob::i18n_domain ),
			__( 'Themes', WpTomob::i18n_domain ),
			$capability_required,
			self::menu_item,
			array( __CLASS__, 'themes_page' )
		);
	}

	public static function admin_enqueue_scripts() {
		global $plugin_page;

		if( $plugin_page != self::menu_item ) {
			return;
		}

		$suffix = SCRIPT_DEBUG ? '' : '.min';

		wp_enqueue_script( 'wpak-theme', plugins_url( 'lib/themes/themes' . $suffix . '.js', dirname( dirname( __FILE__ ) ) ), array( 'wp-backbone', 'wp-a11y' ), WpTomob::resources_version, true );
		  	}





	public static function admin_body_class( $class ) {
		// Fake themes.css stylesheet to make some "theme.php" specific styles work on our page as well
		return $class . ' themes-php';
	}

	public static function themes_page() {
		require_once 'themes-page-template.php';
	}

	public static function inner_main_infos_box( $post, $current_box ) {
		$available_themes = WpakThemes::get_available_themes(true);
		$current_theme = WpakThemesStorage::get_current_theme( $post->ID );
		$main_infos = WpakApps::get_app_main_infos( $post->ID );
		$color_infos = WpakApps::get_app_color_infos( $post->ID );
		?>
		<a href="#" class="hide-if-no-js wptm_help"><?php _e( 'Help me', WpTomob::i18n_domain ); ?></a>
		<?php if ( !empty($available_themes) ): ?>
			<label><?php _e( 'Choose theme', WpTomob::i18n_domain ) ?> : </label>
<div class="row">
	<img id="01" class="layout_icons" style="height: 50px;width: 70px;margin: 5px;cursor:pointer;" src="<?php echo plugins_url('images', dirname( dirname( __FILE__ ) )).'/lyout/01.png';?>">
	<img id="02" class="layout_icons" style="height: 50px;width: 70px;margin: 5px;cursor:pointer;" src="<?php echo plugins_url('images', dirname( dirname( __FILE__ ) )).'/lyout/02.png';?>">
	<img id="03" class="layout_icons" style="height: 50px;width: 70px;margin: 5px;cursor:pointer;" src="<?php echo plugins_url('images', dirname( dirname( __FILE__ ) )).'/lyout/03.png';?>">
	<img id="04" class="layout_icons" style="height: 50px;width: 70px;margin: 5px;cursor:pointer;" src="<?php echo plugins_url('images', dirname( dirname( __FILE__ ) )).'/lyout/04.png';?>">
	<img id="05" class="layout_icons" style="height: 50px;width: 70px;margin: 5px;cursor:pointer;" src="<?php echo plugins_url('images', dirname( dirname( __FILE__ ) )).'/lyout/05.png';?>">
	<img id="06" class="layout_icons" style="height: 50px;width: 70px;margin: 5px;cursor:pointer;" src="<?php echo plugins_url('images', dirname( dirname( __FILE__ ) )).'/lyout/06.png';?>">
	<img id="07" class="layout_icons" style="height: 50px;width: 70px;margin: 5px;cursor:pointer;" src="<?php echo plugins_url('images', dirname( dirname( __FILE__ ) )).'/lyout/07.png';?>">
	<img id="08" class="layout_icons" style="height: 50px;width: 70px;margin: 5px;cursor:pointer;" src="<?php echo plugins_url('images', dirname( dirname( __FILE__ ) )).'/lyout/08.png';?>">
</div>
		 
			<select name="wptm_app_theme_choice" id="wptm_app_theme_choice" autocomplete="off">
				<?php foreach ( $available_themes as $theme_slug => $theme_data ): ?>
					<?php $selected = $theme_slug == $current_theme ? 'selected="selected"' : '' ?>
					<option value="<?php echo esc_attr( $theme_slug ) ?>" <?php echo $selected ?>><?php echo esc_html( $theme_data['Name'] ) ?> </option>
				<?php endforeach ?>
			</select>
			<p class="description"><?php echo esc_html( sprintf( __( 'Choose a theme for your app. Theme defines how your app will look and behave. Themes can be uploaded in the Upload Themes panel. Themes are stored in the %s folder of your WordPress install.', WpTomob::i18n_domain ), basename(WP_CONTENT_DIR) .'/'. WpakThemes::themes_directory ) ) ?></p>
		<?php else: ?>
			<div class="wptm_no_theme">
				<strong><?php _e( 'No wptomob theme found!', WpTomob::i18n_domain ) ?></strong>
				<br/>
				<?php echo sprintf( __('Please upload a wptomob theme from the "<a href="%s" >Upload Themes</a>" panel or copy a theme directly to the %s directory.', WpTomob::i18n_domain ),
									admin_url('admin.php?page=wptm_bo_upload_themes'),
									basename(WP_CONTENT_DIR) .'/'. WpakThemes::themes_directory
							)
				?>
			</div>
		<?php endif ?>

		<?php foreach ( $available_themes as $theme => $theme_data ): ?>
			<div class="wpak-theme-data" id="wpak-theme-data-<?php echo esc_attr( $theme ) ?>" style="display:none">
				<div class="theme-data-content" style="display:none;">
					<?php echo esc_html( $theme_data['Description'] ) ?>

					<?php
						$theme_meta = array();
						if ( !empty( $theme_data['Version'] ) ) {
							$theme_meta[] = sprintf( __( 'Version %s' ), $theme_data['Version'] );
						}
						if ( !empty( $theme_data['Author'] ) ) {
							$author = $theme_data['Author'];
							if ( !empty( $theme_data['AuthorURI'] ) ) {
								$author = '<a href="' . $theme_data['AuthorURI'] . '">' . $theme_data['Author'] . '</a>';
							}
							$theme_meta[] = sprintf( __( 'By %s' ), $author );
						}
						if ( ! empty( $theme_data['ThemeURI'] ) ) {
							$theme_meta[] = sprintf( '<a href="%s">%s</a>',
								esc_url( $theme_data['ThemeURI'] ),
								__( 'Visit theme site' )
							);
						}
						if ( ! empty( $theme_data['WpakVersionRequired'] ) ) {
							$theme_meta[] = sprintf( __( 'Requires wptomob version %s', WpTomob::i18n_domain ), $theme_data['WpakVersionRequired'] );
						}
					?>

					<?php if( !empty($theme_meta) ): ?>
						<div class="theme-meta-data"><?php echo implode(' | ',$theme_meta) ?></div>
					<?php endif ?>
				</div>
			</div>
		<?php endforeach ?>

		
		<div >
			<div class="tab-pane active" id="Style" style="overflow-x: visible;">
                    
                    <!--COLOR PICKERS-->
                    <div class="row">
                        <div class="span3 offset1 col-sm-5">
                            <label for="ThemeAColorHex">Menu Background Color</label>
                            <div id="menuAColorHex" class="input-append color colorpicker123 app-attribute-color" style="display: block;" data-color="#000000" data-color-format="hex" data-colorpicker-guid="1">
                                <input name="wptm_custom_menu_back" class="form-" value="<?php echo esc_attr( $color_infos['wptm_custom_menu_back'] ) ?>" type="text">
                            </div>
                                
                            <label for="BColorHex">Menu Text Color</label>
                            <div id="menuBColorHex" class="input-append color colorpicker123 app-attribute-color" style="display: block;" data-color="#ffffff" data-color-format="hex" data-colorpicker-guid="2">
                                <input name="wptm_custom_menu_text_color" class="form-" value="<?php echo esc_attr( $color_infos['wptm_custom_menu_text_color'] ) ?>" type="text">
                            </div>
                            
                            <label for="HeaderFontColorHex">Header Text Color</label>
                            <div id="HeaderFontColorHex" class="input-append color colorpicker123 app-attribute-color" style="display: block;" data-color="#ffffff" data-color-format="hex" data-colorpicker-guid="3">
                                <input name="wptm_custom_header_text_color" class="form-" value="<?php echo esc_attr( $color_infos['wptm_custom_header_text_color'] ) ?>" type="text">
                            </div> 

                            <label for="titleFontColorHex">Post Title Color</label>
                            <div id="titleFontColorHex" class="input-append color colorpicker123 app-attribute-color" style="display: block;" data-color="#ffffff" data-color-format="hex" data-colorpicker-guid="3">
                                <input name="wptm_custom_post_title_color" class="form-" value="<?php echo esc_attr( $color_infos['wptm_custom_post_title_color'] ) ?>" type="text">
                            </div>
                        </div>
                        <div class="span3  col-sm-5">
                            <label for="PostBgColorHex">Menu Items Background Color</label>
                            <div id="menuitemsColorHex" class="input-append color colorpicker123 app-attribute-color" style="display: block;" data-color="#ffffff" data-color-format="hex" data-colorpicker-guid="4">
                                <input name="wptm_custom_menu_items_colo" class="form-" value="<?php echo esc_attr( $color_infos['wptm_custom_menu_items_colo'] ) ?>" type="text"> 
                            </div>
                            
                            <label for="PostFontColorHex">Application Background  Color</label>
                            <div id="appbackColorHex" class="input-append color colorpicker123 app-attribute-color" style="display: block;" data-color="#000000" data-color-format="hex" data-colorpicker-guid="5">
                                <input name="wptm_custom_application_background" class="form-" value="<?php echo esc_attr( $color_infos['wptm_custom_application_background'] ) ?>" type="text">  
                            </div>
                                
                            <label for="HeaderBgColorHex">Header Background Color</label>
                            <div id="HeaderBgColorHex" class="input-append color colorpicker123 app-attribute-color" style="display: block;" data-color="#000000" data-color-format="hex" data-colorpicker-guid="6">
                                <input name="wptm_custom_header_background" class="form-" value="<?php echo esc_attr( $color_infos['wptm_custom_header_background'] ) ?>" type="text">
                            </div>
                        </div>
                    </div>

                    <!--LAYOUT PICKER-->
                  
                    
                    

                </div>

                
		</div>

		 <?php 
 wp_enqueue_script( 'custom_js', plugins_url( 'components/js/bootstrap-colorpicker.js', dirname( __FILE__ ) ));
 wp_enqueue_script( 'jquery_easing', 'http://thecodeplayer.com/uploads/js/jquery.easing.min.js');


    wp_register_style( 'my_css',    plugins_url( 'components/css/bootstrap-colorpicker.css',    dirname( __FILE__ ) ));
    wp_register_style( 'bootstrap',    'https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css');
    wp_enqueue_style ( 'bootstrap' );
    wp_enqueue_style ( 'my_css' );

            ?>
            <style>
.layout_icons:hover{
	border: 1px solid black;
transform: scale(1.4,1.4);
margin: 0px 20px;
}
.layout_icons.selected{
	border: 1px solid black;
	background: #27AE60;
}
.platform_icons:hover{
transform: scale(1.4,1.4);
margin: 0px 20px;
}
.platform_icons.selected{
	background: #D5D7D6;
border-radius: 9px;
}
#osImagep {
	position: absolute;
width: 255px;
height: 545px;
top: -36px;
}
#osImage {
    /*-moz-transform: scale(0.8);
    -o-transform: scale(0.8);
    -webkit-transform: scale(0.8);
    -ms-transform: scale(0.8);*/
    position: absolute;
    width: 259px;
    height: 475px;
}
#osImagesplash {
 position: absolute;
width: 210px;
height: 382px;
top: 49px;
left: 37px;
}
iframe#preview.android {
    -moz-transform: scale(0.7);
    -o-transform: scale(0.7);
    -webkit-transform: scale(0.7);
    -ms-transform: scale(0.7);
 position: absolute;
    width: 297px;
    height: 513px;
    border: none !important;
    left: -17px;
    top: -20px;
    z-index: 2;

}
#preview1 {
-moz-transform: scale(0.7);
-o-transform: scale(0.7);
-webkit-transform: scale(0.7);
-ms-transform: scale(0.7);
position: absolute;
width: 325px;
height: 615px;
border: none !important;
left: -34px;
top: -69px;
z-index: 2;
}
            </style>
		<?php wp_nonce_field( 'wpak-theme-data-' . $post->ID, 'wpak-nonce-theme-data' ) ?>

		<?php // TODO: put this style in an external file ?>
		<style>
			.wpak-theme-data{ padding:9px 12px; margin-bottom: 10px }
			.theme-data-content{ margin-top: 0 }
			.wpak-app-title{ margin-top: 15px; border-top: 1px solid #ddd; padding-top:10px }
			.theme-meta-data{ margin-top: 7px }
			.wptm_no_theme{ text-align: center; font-size:120%; line-height: 2em; margin:30px }
		</style>

		<script>
			(function(){
				var $ = jQuery;
				$('#wptm_app_theme_choice').change(function(){
					$('.wpak-theme-data').hide();
					var theme = this.value;
					$('#wpak-theme-data-'+ theme).show();
				});
				$('#wptm_app_theme_choice').change();
			})();
			jQuery(document).ready(function(){
				jQuery("#wptm_app_theme_choice").hide();
	classpl=jQuery("#wptm_app_theme_choice").val();
	jQuery('#'+classpl).addClass('selected')
				jQuery('.layout_icons').click(function(){
					jQuery('.layout_icons').removeClass('selected')
					jQuery(this).addClass('selected')
					id=jQuery(this).attr('id')
					jQuery("#wptm_app_theme_choice option[value="+id+"]").attr('selected','selected')

  var theme= jQuery("#wptm_app_theme_choice").val()
    var iframe = document.getElementById("theme"+theme);
    var style='<link rel="stylesheet" href="'+"<?php echo get_site_url(); ?>/wp-content/themes-wptomob/"+theme+"/css/common.css?wptm_app_id=gfdgfd"+'">'+
    '<link rel="stylesheet" href="'+"<?php echo get_site_url(); ?>/wp-content/themes-wptomob/"+theme+"/css/post-list.css?wptm_app_id=gfdgfd"+'">'+
    '<link rel="stylesheet" href="'+"<?php echo get_site_url(); ?>/wp-content/themes-wptomob/"+theme+"/css/post-detail.css?wptm_app_id=gfdgfd"+'">';
    var iframe_contents = style+iframe.innerHTML;
    jQuery('#preview1').html(iframe_contents)
    
            jQuery('#app-header').css({'color':jQuery('#HeaderFontColorHex input').val()})
            
            jQuery('.post_text h2').css({'color':jQuery('#titleFontColorHex input').val()})
        
            jQuery('#app-header').css({'background':jQuery('#HeaderBgColorHex input').val()})
          jQuery('#app-menu').css({'background':jQuery('#menuAColorHex input').val()})
        
            jQuery('#preview1 #content').css({'background':jQuery("#appbackColorHex input").val()})
            jQuery('#menu-items.menu-items li a').css({'background':jQuery('#menuitemsColorHex input').val()})
      
            jQuery('#menu-items.menu-items li a').css({'color':jQuery('#menuBColorHex input').val()})
					})
				jQuery('#appbackColorHex input').change(function(){
						jQuery('#preview1 #content').css({'background':jQuery(this).val()})
				})
				jQuery('#menuAColorHex input').change(function(){
						jQuery('#app-menu').css({'background':jQuery(this).val()})
				})
				jQuery('#menuitemsColorHex input').change(function(){
						jQuery('#menu-items.menu-items li a').css({'background':jQuery(this).val()})
				})
				jQuery('#menuBColorHex input').change(function(){
						jQuery('#menu-items.menu-items li a').css({'color':jQuery(this).val()})
				})
				jQuery('#titleFontColorHex input').change(function(){
						jQuery('.post_text h2').css({'color':jQuery(this).val()})
				})
				jQuery('#HeaderFontColorHex input').change(function(){
						jQuery('#app-header').css({'color':jQuery(this).val()})
				})
				jQuery('#HeaderBgColorHex input').change(function(){
						jQuery('#app-header').css({'background':jQuery(this).val()})
				})
				jQuery('.colorpicker123 input').colorpicker();
			})
		</script>

		<?php
		
	}

	public static function save_post( $post_id ) {

		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}

		if ( empty( $_POST['post_type'] ) || $_POST['post_type'] != 'wptm_apps' ) {
			return;
		}

		if ( !current_user_can( 'edit_post', $post_id ) && !current_user_can( 'wptm_edit_apps', $post_id ) ) {
			return;
		}

		if ( !check_admin_referer( 'wpak-theme-data-' . $post_id, 'wpak-nonce-theme-data' ) ) {
			return;
		}

		if ( isset( $_POST['wptm_app_title'] ) ) {
			update_post_meta( $post_id, '_wptm_app_title', sanitize_text_field( $_POST['wptm_app_title'] ) );
		}

		if ( isset( $_POST['wptm_custom_menu_back'] ) ) {
			update_post_meta( $post_id, '_wptm_custom_menu_back', sanitize_text_field( $_POST['wptm_custom_menu_back'] ) );
		}

		if ( isset( $_POST['wptm_custom_menu_text_color'] ) ) {
			update_post_meta( $post_id, '_wptm_custom_menu_text_color', sanitize_text_field( $_POST['wptm_custom_menu_text_color'] ) );
		}

		if ( isset( $_POST['wptm_custom_header_text_color'] ) ) {
			update_post_meta( $post_id, '_wptm_custom_header_text_color', sanitize_text_field( $_POST['wptm_custom_header_text_color'] ) );
		}

		if ( isset( $_POST['wptm_custom_post_title_color'] ) ) {
			update_post_meta( $post_id, '_wptm_custom_post_title_color', sanitize_text_field( $_POST['wptm_custom_post_title_color'] ) );
		}

		if ( isset( $_POST['wptm_custom_menu_items_colo'] ) ) {
			update_post_meta( $post_id, '_wptm_custom_menu_items_colo', sanitize_text_field( $_POST['wptm_custom_menu_items_colo'] ) );
		}

		if ( isset( $_POST['wptm_custom_application_background'] ) ) {
			update_post_meta( $post_id, '_wptm_custom_application_background', sanitize_text_field( $_POST['wptm_custom_application_background'] ) );
		}

		if ( isset( $_POST['wptm_custom_header_background'] ) ) {
			update_post_meta( $post_id, '_wptm_custom_header_background', sanitize_text_field( $_POST['wptm_custom_header_background'] ) );
		}

		if ( isset( $_POST['wptm_app_theme_choice'] ) ) {
			WpakThemesStorage::set_current_theme( $post_id, $_POST['wptm_app_theme_choice'] );
		}
	}

}

WpakThemesBoSettings::hooks();
