<?php
/*
Plugin Name: wptomob
Plugin URI:  https://github.com/uncatcrea/wptomob
Description: Build Phonegap Mobile apps based on your WordPress content.
Version:     1.0.2
Author:      Uncategorized Creations
Author URI:  http://getWpTomob.com
Text Domain: wptomob
Domain Path: /lang
License:     GPL-2.0+
License URI: http://www.gnu.org/licenses/gpl-2.0.txt
Copyright:   2013-2016 Uncategorized Creations

This plugin, like WordPress, is licensed under the GPL.
Use it to make something cool, have fun, and share what you've learned with others.
*/

if ( !class_exists( 'WpTomob' ) ) {

	class WpTomob {

		const resources_version = '1.0';
		const i18n_domain = 'wptomob';

		public static function hooks() {
			add_action( 'plugins_loaded', array( __CLASS__, 'plugins_loaded' ) );

			register_activation_hook( __FILE__, array( __CLASS__, 'on_activation' ) );
			register_deactivation_hook( __FILE__, array( __CLASS__, 'on_deactivation' ) );

			add_action( 'init', array( __CLASS__, 'init' ) );
			add_action( 'template_redirect', array( __CLASS__, 'template_redirect' ), 5 );

			add_action( 'admin_notices', array( __CLASS__, 'admin_notices' ) );
			add_action( 'admin_init', array( __CLASS__, 'upgrade' ) );
		}

		protected static function lib_require() {
			require_once(dirname( __FILE__ ) . '/lib/addons/addons.php');
			require_once(dirname( __FILE__ ) . '/lib/user-permissions/user-login.php');
			require_once(dirname( __FILE__ ) . '/lib/web-services/web-services.php');
			require_once(dirname( __FILE__ ) . '/lib/apps/apps.php');
			require_once(dirname( __FILE__ ) . '/lib/apps/build.php');
			require_once(dirname( __FILE__ ) . '/lib/themes/themes.php');
			require_once(dirname( __FILE__ ) . '/lib/themes/upload-themes.php');
			require_once(dirname( __FILE__ ) . '/lib/user-permissions/user-permissions.php');
			require_once(dirname( __FILE__ ) . '/lib/settings/settings.php');
			require_once(dirname( __FILE__ ) . '/lib/builder/builder.php');
			require_once(dirname( __FILE__ ) . '/lib/components/components.php');
			require_once(dirname( __FILE__ ) . '/lib/navigation/navigation.php');
			require_once(dirname( __FILE__ ) . '/lib/options/options.php');
			require_once(dirname( __FILE__ ) . '/lib/simulator/simulator.php');
			require_once(dirname( __FILE__ ) . '/lib/simulator/server-rewrite.php');
			require_once(dirname( __FILE__ ) . '/lib/shortcodes/show_hide_in_apps.php');
		}

		public static function plugins_loaded() {
			load_plugin_textdomain( self::i18n_domain, false, dirname( plugin_basename( __FILE__ ) ) . '/lang/' );
			self::lib_require();
		}

		public static function on_activation() {
			self::lib_require();

			self::add_phonegap_table();
			self::add_rewrite_rules();
			flush_rewrite_rules();

			$directory = WpakThemes::create_theme_directory();
			if( !empty( $directory ) ) {
				WpakThemes::install_default_themes();
			}

			//If WordPress Network, add wptomob custom rewrite rules to htacces,
			//required for apps' preview to work.
			if( is_multisite() ) {
				WpakServerRewrite::prepend_wp_network_wptm_rules_to_htaccess();
			}
		}

		public static function on_deactivation( $network_wide ) {
			flush_rewrite_rules();

			//If this is a network wide uninstall, we can remove wptomob rules.
			//If this is not a network wide uninstall, we can't remove our rules
			//as they may be needed by another site of the network. We could
			//check all sites to see if wptomob is installed somewhere, but this would
			//require making switch_to_blog() for every sites, which we consider
			//dangerous performance-wise, knowing that we can have > 10000 websites...
			//So, unless we find a better solution, we don't remove our rules
			//if this is not a network wide uninstall. (This should not cause any
			//problem as wptomob rules only apply to wptomob plugin
			//and can't interfer with other rules).
			if( is_multisite() && $network_wide ) {
				WpakServerRewrite::delete_wp_network_wptm_rules_from_htaccess();
			}

		}

		public static function init() {
			self::add_rewrite_rules();
			WpakComponents::handle_images_sizes();
		}

		public static function template_redirect() {
			WpakWebServices::template_redirect();
		}

		protected static function add_phonegap_table() {
			global $wpdb;
			$charset_collate = $wpdb->get_charset_collate();
				require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
				$wptm_phonegap_table = "
		CREATE TABLE {$wpdb->prefix}wptm_phonegap_table (
			id bigint(12) unsigned NOT NULL auto_increment,
			app_id bigint(12) NOT NULL default 0,
			post_id bigint(12) NOT NULL default 0,
			title VARCHAR(128) NOT NULL default '',
			description VARCHAR(255) NOT NULL default '',
			icon VARCHAR(255) NOT NULL default '',
			version VARCHAR(6) NOT NULL default '',
			app_url VARCHAR(255) NOT NULL default '',
			platform VARCHAR(30) NOT NULL DEFAULT '' ,
			PRIMARY KEY (id),
			KEY id (id)
		) $charset_collate; 
	";
	dbDelta($wptm_phonegap_table);
	$wptm_keystore_table = "
		CREATE TABLE {$wpdb->prefix}wptm_keystore_table (
			id bigint(12) unsigned NOT NULL auto_increment,
			key_id bigint(12) NOT NULL default 0,
			key_password VARCHAR(128) NOT NULL default '',
			alais VARCHAR(128) NOT NULL default '',
			alais_password VARCHAR(255) NOT NULL default '',
			platform VARCHAR(30) NOT NULL DEFAULT '' ,
			PRIMARY KEY (id),
			KEY id (id)
		) $charset_collate; 
	";
	dbDelta($wptm_keystore_table);
		}
		protected static function add_rewrite_rules() {
			WpakWebServices::add_rewrite_tags_and_rules();
			WpakConfigFile::rewrite_rules();
			WpakThemes::rewrite_rules();
			WpakWpCoreJsFiles::rewrite_rules();
		}

		/**
		 * Check if wptomob Plugin has been upgraded, and run corresponding upgrade routine(s).
		 *
		 * Contains conditional checks to determine which upgrade scripts to run,
		 * based on database version and wptomob version being updated-to.
		 */
		public static function upgrade() {
			$db_version = get_option( 'wptm_version', null );
			$data = get_file_data( __FILE__, array( 'version' => 'Version' ) );
			$plugin_file_version = $data['version'];

			// /!\ NOTE
			//version_compare() considers 1 != 1.0 != 1.0.0
			//-> it considers: 1 < 1.0 < 1.0.0
			//So for example if we have 1.0 in plugin file we should compare it to 1.0, not 1.0.0
			
			if ( $db_version === null ) {
				//This is a first install, no need for upgrade routine.
				//Just set db version:
				update_option( 'wptm_version', $plugin_file_version );
				
				//And, if network install, save rewrite rules because on_activation hook
				//is not executed when network activating a plugin. (See notes here: 
				//https://codex.wordpress.org/Function_Reference/register_activation_hook )
				if ( is_multisite() ) {
					self::add_rewrite_rules();
					flush_rewrite_rules();
				}
				return;
			}
			
			if ( version_compare( $plugin_file_version, $db_version, '=' ) ) {
				// We are up-to-date. Nothing to do.
				return;
			}

			if( version_compare( $db_version, '0.6', '<' ) ) {
				self::upgrade_060();
			}
			
			if( version_compare( $db_version, '1.0', '<' ) ) {
				self::upgrade_100();
			}
			
			if ( !version_compare( $plugin_file_version, $db_version, '=' ) ) {
				//Update db version not to run update scripts again and so that
				//db version is up to date:
				update_option( 'wptm_version', $plugin_file_version );
			}
			
		}

		/**
		 * Execute changes made in wptomob 0.6.0
		 */
		protected static function upgrade_060() {
			// Remove 'wptm_used_themes' transient since its value's structure changed with this version
			delete_transient( 'wptm_used_themes' );
			
			//Memorize we've gone that far successfully, to not re-run this routine 
			//in case something goes wrong in next upgrade routines:
			update_option( 'wptm_version', '0.6' );
		}
		
		/**
		 * Execute changes made in wptomob 1.0
		 */
		protected static function upgrade_100() {
			//We have to flush rewrite rules to take into account new rules to use WP core's versions
			//of jQuery, undercore and backbone:
			self::add_rewrite_rules();
			flush_rewrite_rules();
			
			//If WordPress Network, add wptomob custom rewrite rules to htacces,
			//required for apps' preview to work.
			if( is_multisite() ) {
				WpakServerRewrite::prepend_wp_network_wptm_rules_to_htaccess();
			}
			
			//Memorize we've gone that far successfully, to not re-run this routine 
			//in case something goes wrong in next upgrade routines:
			update_option( 'wptm_version', '1.0' );
		}

		/**
		 * If permalinks are not activated, send an admin notice
		 */
		public static function admin_notices() {
			if ( !get_option( 'permalink_structure' ) ) {
				?>
				<div class="error">
					<p>
						<?php
							_e( 'wptomob requires WordPress permalinks to be activated: '
								. 'see the <a href="http://codex.wordpress.org/Using_Permalinks#Choosing_your_permalink_structure">"Using permalink" Codex section</a> '
								. 'for more info about how to activate permalinks.',
								WpTomob::i18n_domain
							);
						?>
					</p>
				</div>
				<?php
			}
		}
	}

	WpTomob::hooks();

	/**
	 * wptomob WP CLI commands :
	 */
	if ( defined('WP_CLI') && WP_CLI ) {
		require_once( dirname( __FILE__ ) .'/wp-cli-commands/wpak-commands.php' );
	}
}
